import { Engine, EngineResult, Rule } from 'json-rules-engine';
import { GlobalRules, S3GetInput } from './types/UtilTypes';
import { S3 } from './libs/s3';
import { Logger } from '@aws-lambda-powertools/logger';
import { CoreData } from './http/coreData';

const logger = new Logger({ serviceName: 'ApplyRules' });

export async function applyRules(inputData: GlobalRules): Promise<any[]> {
  try {
    const bucketName = process.env.BUCKET_JSON_RULES;

    const param: S3GetInput = {
      Bucket: bucketName,
      Key: inputData.nameRule,
      ...(inputData.versionNameRule && {
        VersionId: inputData.versionNameRule,
      }),
    };

    const s3Data = await S3.getObject(param);
    const jsonRules: Rule[] = JSON.parse(s3Data).decisions;

    const engine = new Engine();
    jsonRules.forEach((rule) => {
      engine.addRule(rule);
    });

    let facts = inputData?.data;

    if (inputData?.dynamicFact?.accountId) {
      const urlDynamicFact = process.env.URL_DYNAMIC_FACT;

      const dynamicData = await CoreData.getInformation(
        inputData.dynamicFact.accountId,
        urlDynamicFact
      );
      engine.addFact(inputData?.dynamicFact.name, async () => dynamicData);
      facts = { accountId: inputData?.dynamicFact.accountId };
    }

    const results: EngineResult = await engine.run(facts);

    return [results];
  } catch (error) {
    logger.error(error);
    throw error;
  }
}
